@@ -58,10 +58,7 @@ auto register_histogram(py::module& m, const char* name, const char* desc) {
                  return a;
              })
 
-        .def(py::self + py::self)
-        // .def(py::self + value_type())
         .def(py::self += py::self)
-        // .def(py::self += value_type())
 
         .def("__eq__",
              [](const histogram_t& self, const py::object& other) {
@@ -90,22 +87,21 @@ auto register_histogram(py::module& m, const char* name, const char* desc) {
 
         ;
 
-    // Atomics for example do not support these operations
+// Protection against an overzealous warning system
+// https://bugs.llvm.org/show_bug.cgi?id=43124
+#ifdef __clang__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wself-assign-overloaded"
+#endif
     def_optionally(hist,
-                   bh::detail::has_operator_rmul<histogram_t, double>{},
-                   py::self *= double());
+                   bh::detail::has_operator_rdiv<histogram_t, histogram_t>{},
+                   py::self /= py::self);
     def_optionally(hist,
-                   bh::detail::has_operator_rmul<histogram_t, double>{},
-                   py::self * double());
-    def_optionally(hist,
-                   bh::detail::has_operator_rmul<histogram_t, double>{},
-                   double() * py::self);
-    def_optionally(hist,
-                   bh::detail::has_operator_rdiv<histogram_t, double>{},
-                   py::self /= double());
-    def_optionally(hist,
-                   bh::detail::has_operator_rdiv<histogram_t, double>{},
-                   py::self / double());
+                   bh::detail::has_operator_rmul<histogram_t, histogram_t>{},
+                   py::self *= py::self);
+#ifdef __clang__
+#pragma GCC diagnostic pop
+#endif
 
     hist.def(
             "to_numpy",
