@@ -1,18 +1,31 @@
+# -*- coding: utf-8 -*-
 from __future__ import absolute_import, division, print_function
 
-del absolute_import, division, print_function
-
-from .._core.axis import options
-
 from .._core import axis as ca
 
 from .kwargs import KWArgs
 from .sig_tools import inject_signature
 from .axis_transform import AxisTransform
-from .utils import cast, register, set_family, MAIN_FAMILY, CPP_FAMILY, set_module
+from .utils import cast, register, set_family, MAIN_FAMILY, set_module
+from .six import string_types
 
 import copy
 
+del absolute_import, division, print_function
+
+
+def _isstr(value):
+    """
+    Check to see if this is a stringlike or a (nested) iterable of stringlikes
+    """
+
+    if isinstance(value, string_types + (bytes,)):
+        return True
+    elif hasattr(value, "__iter__"):
+        return all(_isstr(v) for v in value)
+    else:
+        return False
+
 
 # Contains common methods and properties to all axes
 @set_module("boost_histogram.axis")
@@ -29,7 +42,14 @@ class Axis(object):
         Return the fractional index(es) given a value (or values) on the axis.
         """
 
-        return self._ax.index(value)
+        if not _isstr(value):
+            return self._ax.index(value)
+        else:
+            raise TypeError(
+                "index({value}) cannot be a string for a numerical axis".format(
+                    value=value
+                )
+            )
 
     def value(self, index):
         """
@@ -64,6 +84,10 @@ class Axis(object):
     def metadata(self, value):
         self._ax.metadata = value
 
+    @metadata.deleter
+    def metadata(self):
+        self._ax.metadata = None
+
     @classmethod
     def _convert_cpp(cls, cpp_object):
         nice_ax = cls.__new__(cls)
@@ -76,11 +100,22 @@ class Axis(object):
     def __iter__(self):
         return self._ax.__iter__()
 
+    def _process_loc(self, start, stop):
+        """
+        Compute start and stop into actual start and stop values in Boost.Histogram.
+        None -> -1 or 0 for start, -> len or len+1 for stop. If start or stop are
+        callable, then call them with the axes.
+        """
 
-# Mixin for main style classes
-# Contains common methods and properties to all Main module axes
-class MainAxisMixin(object):
-    __slots__ = ()
+        def _process_internal(item, default):
+            return default if item is None else item(self) if callable(item) else item
+
+        begin = _process_internal(start, -1 if self._ax.options.underflow else 0)
+        end = _process_internal(
+            stop, len(self) + (1 if self._ax.options.overflow else 0)
+        )
+
+        return begin, end
 
     def __repr__(self):
         return "{self.__class__.__name__}({args}{kwargs})".format(
@@ -170,52 +205,6 @@ class MainAxisMixin(object):
         return self._ax.widths
 
 
-# Contains all common methods for cpp module axes
-class CppAxisMixin(object):
-    __slots__ = ()
-
-    def __repr__(self):
-        return repr(self._ax)
-
-    def options(self):
-        """
-        Return the options.  Fields:
-          .underflow - True if axes captures values that are too small
-          .overflow  - True if axes captures values that are too large
-                       (or non-valid for category axes)
-          .growth    - True if axis can grow
-          .circular  - True if axis wraps around
-        """
-        return self._ax.options
-
-    def size(self):
-        """
-        Return number of bins excluding under- and overflow.
-        """
-        return self._ax.size
-
-    def extent(self):
-        """
-        Return number of bins including under- and overflow.
-        """
-        return self._ax.extent
-
-    def edges(self):
-        return self._ax.edges
-
-    def centers(self):
-        """
-        An array of bin centers.
-        """
-        return self._ax.centers
-
-    def widths(self):
-        """
-        An array of bin widths.
-        """
-        return self._ax.widths
-
-
 # Contains all common methods and properties for Regular axes
 @register(
     {
@@ -230,7 +219,9 @@ class CppAxisMixin(object):
         ca.regular_circular,
     }
 )
-class BaseRegular(Axis):
+@set_module("boost_histogram.axis")
+@set_family(MAIN_FAMILY)
+class Regular(Axis):
     __slots__ = ()
 
     @inject_signature(
@@ -306,12 +297,6 @@ class BaseRegular(Axis):
         else:
             raise KeyError("Unsupported collection of options")
 
-
-@set_module("boost_histogram.axis")
-@set_family(MAIN_FAMILY)
-class Regular(BaseRegular, MainAxisMixin):
-    __slots__ = ()
-
     def _repr_args(self):
         "Return inner part of signature for use in repr"
 
@@ -334,17 +319,6 @@ class Regular(BaseRegular, MainAxisMixin):
         return None
 
 
-@set_module("boost_histogram.cpp.axis")
-@set_family(CPP_FAMILY)
-class regular(BaseRegular, CppAxisMixin):
-    __slots__ = ()
-
-    def transform(self):
-        if hasattr(self._ax, "transform"):
-            return cast(self, self._ax.transform, AxisTransform)
-        return None
-
-
 @register(
     {
         ca.variable_none,
@@ -355,7 +329,9 @@ class regular(BaseRegular, CppAxisMixin):
         ca.variable_circular,
     }
 )
-class BaseVariable(Axis):
+@set_family(MAIN_FAMILY)
+@set_module("boost_histogram.axis")
+class Variable(Axis):
     __slots__ = ()
 
     @inject_signature(
@@ -410,12 +386,6 @@ class BaseVariable(Axis):
         else:
             raise KeyError("Unsupported collection of options")
 
-
-@set_family(MAIN_FAMILY)
-@set_module("boost_histogram.axis")
-class Variable(BaseVariable, MainAxisMixin):
-    __slots__ = ()
-
     def _repr_args(self):
         "Return inner part of signature for use in repr"
 
@@ -425,12 +395,6 @@ class Variable(BaseVariable, MainAxisMixin):
             return "[{}]".format(", ".join(format(v, "g") for v in self.edges))
 
 
-@set_family(CPP_FAMILY)
-@set_module("boost_histogram.cpp.axis")
-class variable(BaseVariable, CppAxisMixin):
-    __slots__ = ()
-
-
 @register(
     {
         ca.integer_none,
@@ -441,7 +405,9 @@ class variable(BaseVariable, CppAxisMixin):
         ca.integer_circular,
     }
 )
-class BaseInteger(Axis):
+@set_family(MAIN_FAMILY)
+@set_module("boost_histogram.axis")
+class Integer(Axis):
     __slots__ = ()
 
     @inject_signature(
@@ -496,98 +462,13 @@ class BaseInteger(Axis):
         else:
             raise KeyError("Unsupported collection of options")
 
-
-@set_family(MAIN_FAMILY)
-@set_module("boost_histogram.axis")
-class Integer(BaseInteger, MainAxisMixin):
-    __slots__ = ()
-
     def _repr_args(self):
         "Return inner part of signature for use in repr"
 
         return "{start:g}, {stop:g}".format(start=self.edges[0], stop=self.edges[-1])
 
 
-@set_family(CPP_FAMILY)
-@set_module("boost_histogram.cpp.axis")
-class integer(BaseInteger, CppAxisMixin):
-    __slots__ = ()
-
-
-@register({ca.category_str_growth, ca.category_str})
-class BaseStrCategory(Axis):
-    __slots__ = ()
-
-    @inject_signature("self, categories, *, metadata=None, growth=False")
-    def __init__(self, categories, **kwargs):
-        """
-        Make a category axis with strings; items will
-        be added to a predefined list of bins or a growing (with growth=True)
-        list of bins.
-
-
-        Parameters
-        ----------
-        categories : Iterator[str]
-            The bin values in strings. May be empty if growth is enabled.
-        metadata : object
-            Any Python object to attach to the axis, like a label.
-        growth : bool = False
-            Allow the axis to grow if a value is encountered out of range.
-            Be careful, the axis will grow as large as needed.
-        """
-        with KWArgs(kwargs) as k:
-            metadata = k.optional("metadata")
-            options = k.options(growth=False)
-
-        # We need to make sure we support Python 2 for now :(
-        # henryiii: This shortcut possibly should be removed
-        if isinstance(categories, (type(""), type(u""))):
-            categories = list(categories)
-
-        if options == {"growth"}:
-            self._ax = ca.category_str_growth(categories, metadata)
-        elif options == set():
-            self._ax = ca.category_str(categories, metadata)
-        else:
-            raise KeyError("Unsupported collection of options")
-
-
-@register({ca.category_int, ca.category_int_growth})
-class BaseIntCategory(Axis):
-    __slots__ = ()
-
-    @inject_signature("self, categories, *, metadata=None, growth=False")
-    def __init__(self, categories, **kwargs):
-        """
-        Make a category axis with ints; items will
-        be added to a predefined list of bins or a growing (with growth=True)
-        list of bins. An empty list is allowed if growth=True.
-
-
-        Parameters
-        ----------
-        categories : Iteratable[int]
-            The bin values, either ints or strings.
-        metadata : object
-            Any Python object to attach to the axis, like a label.
-        growth : bool = False
-            Allow the axis to grow if a value is encountered out of range.
-            Be careful, the axis will grow as large as needed.
-        """
-        with KWArgs(kwargs) as k:
-            metadata = k.optional("metadata")
-            options = k.options(growth=False)
-
-        if options == {"growth"}:
-            self._ax = ca.category_int_growth(categories, metadata)
-        elif options == set():
-            self._ax = ca.category_int(categories, metadata)
-        else:
-            raise KeyError("Unsupported collection of options")
-
-
-class CategoryMixin(object):
+class BaseCategory(Axis):
     __slots__ = ()
 
     def _repr_kwargs(self):
@@ -614,9 +495,56 @@ class CategoryMixin(object):
 
 @set_family(MAIN_FAMILY)
 @set_module("boost_histogram.axis")
-class StrCategory(BaseStrCategory, CategoryMixin, MainAxisMixin):
+@register({ca.category_str_growth, ca.category_str})
+class StrCategory(BaseCategory):
     __slots__ = ()
 
+    @inject_signature("self, categories, *, metadata=None, growth=False")
+    def __init__(self, categories, **kwargs):
+        """
+        Make a category axis with strings; items will
+        be added to a predefined list of bins or a growing (with growth=True)
+        list of bins.
+
+
+        Parameters
+        ----------
+        categories : Iterator[str]
+            The bin values in strings. May be empty if growth is enabled.
+        metadata : object
+            Any Python object to attach to the axis, like a label.
+        growth : bool = False
+            Allow the axis to grow if a value is encountered out of range.
+            Be careful, the axis will grow as large as needed.
+        """
+        with KWArgs(kwargs) as k:
+            metadata = k.optional("metadata")
+            options = k.options(growth=False)
+
+        # henryiii: We currently expand "abc" to "a", "b", "c" - some
+        # Python interfaces protect against that
+
+        if options == {"growth"}:
+            self._ax = ca.category_str_growth(tuple(categories), metadata)
+        elif options == set():
+            self._ax = ca.category_str(tuple(categories), metadata)
+        else:
+            raise KeyError("Unsupported collection of options")
+
+    def index(self, value):
+        """
+        Return the fractional index(es) given a value (or values) on the axis.
+        """
+
+        if _isstr(value):
+            return self._ax.index(value)
+        else:
+            raise TypeError(
+                "index({value}) must be a string or iterable of strings for a StrCategory axis".format(
+                    value=value
+                )
+            )
+
     def _repr_args(self):
         "Return inner part of signature for use in repr"
 
@@ -625,22 +553,88 @@ class StrCategory(BaseStrCategory, CategoryMixin, MainAxisMixin):
 
 @set_family(MAIN_FAMILY)
 @set_module("boost_histogram.axis")
-class IntCategory(BaseIntCategory, CategoryMixin, MainAxisMixin):
+@register({ca.category_int, ca.category_int_growth})
+class IntCategory(BaseCategory):
     __slots__ = ()
 
+    @inject_signature("self, categories, *, metadata=None, growth=False")
+    def __init__(self, categories, **kwargs):
+        """
+        Make a category axis with ints; items will
+        be added to a predefined list of bins or a growing (with growth=True)
+        list of bins. An empty list is allowed if growth=True.
+
+
+        Parameters
+        ----------
+        categories : Iteratable[int]
+            The bin values, either ints or strings.
+        metadata : object
+            Any Python object to attach to the axis, like a label.
+        growth : bool = False
+            Allow the axis to grow if a value is encountered out of range.
+            Be careful, the axis will grow as large as needed.
+        """
+        with KWArgs(kwargs) as k:
+            metadata = k.optional("metadata")
+            options = k.options(growth=False)
+
+        if options == {"growth"}:
+            self._ax = ca.category_int_growth(tuple(categories), metadata)
+        elif options == set():
+            self._ax = ca.category_int(tuple(categories), metadata)
+        else:
+            raise KeyError("Unsupported collection of options")
+
     def _repr_args(self):
         "Return inner part of signature for use in repr"
 
         return "[{0}]".format(", ".join(format(c, "g") for c in self))
 
 
-@set_family(CPP_FAMILY)
-@set_module("boost_histogram.cpp.axis")
-class int_category(BaseIntCategory, CppAxisMixin):
+# Contains all common methods and properties for the boolean axis
+@register({ca.boolean})
+@set_family(MAIN_FAMILY)
+@set_module("boost_histogram.axis")
+class Boolean(Axis):
     __slots__ = ()
 
+    @inject_signature("self, *, metadata=None")
+    def __init__(self, **kwargs):
+        """
+        Make an axis for boolean values.
 
-@set_family(CPP_FAMILY)
-@set_module("boost_histogram.cpp.axis")
-class str_category(BaseStrCategory, CppAxisMixin):
-    __slots__ = ()
+        Parameters
+        ----------
+        metadata : object
+            Any Python object to attach to the axis, like a label.
+        """
+        with KWArgs(kwargs) as k:
+            metadata = k.optional("metadata")
+
+        self._ax = ca.boolean(metadata)
+
+    def _repr_args(self):
+        "Return inner part of signature for use in repr"
+        if self.size == 2:
+            return ""
+        elif self.size == 0:
+            return "<empty>"
+        elif self.size == 1 and self.centers[0] < 0.75:
+            return "<False>"
+        elif self.size == 1:
+            return "<True>"
+        else:
+            # Shouldn't be possible, can't grow
+            "<unknown>"
+
+    def _repr_kwargs(self):
+        """
+        Return options for use in repr. Metadata is last,
+        just in case it spans multiple lines.
+        """
+
+        if self.metadata is not None:
+            return "metadata={0!r}".format(self.metadata)
+        else:
+            return ""
